@@ -9,27 +9,31 @@
 #
 #################################################################
 
+from __future__ import annotations
+from typing import List, Dict, Tuple, Optional, Type, Any, Union, Mapping
 from pylibCZIrw import czi as pyczi
 from czitools import pylibczirw_metadata as czimd
 from czitools import misc
 import numpy as np
-from typing import List, Dict, Tuple, Optional, Type, Any, Union
+from pathlib import Path, PurePath, PureWindowsPath, PurePosixPath
+from typing import List, Dict, Tuple, Optional, Type, Any, Union, Mapping
 from tqdm.contrib.itertools import product
 import dask
 import dask.array as da
+import os
 
 
-def read_6darray(filename: str,
+def read_6darray(filepath: Union[str, os.PathLike[str]],
                  output_order: str = "STCZYX",
                  output_dask: bool = False,
                  chunks_auto: bool = False,
                  remove_adim: bool = True,
-                 **kwargs: int) -> Tuple[Union[np.ndarray, da.Array], czimd.CziMetadata, str]:
+                 **kwargs: int) -> Tuple[Optional[Union[np.ndarray, da.Array]], czimd.CziMetadata, str]:
     """Read a CZI image file as 6D numpy or dask array.
     Important: Currently supported are only scenes with equal size and CZIs with consistent pixel types.
 
     Args:
-        filename (str): filepath for the CZI image
+        filepath (str | Path): filepath for the CZI image
         output_order (str, optional): Order of dimensions for the output array. Defaults to "STCZYX".
         output_dask (bool, optional): If True the output will be a dask array. Defaults to False.
         chunks_auto (bool, optional): Use a chunk size create automatically if True and otherwise use the array shape. Default to False.
@@ -38,15 +42,20 @@ def read_6darray(filename: str,
                                 Will be used to read only a substack from the array
 
     Returns:
-        Tuple[np.ndarray, czimd.Metadata, dim_order6d]: output as 6D numpy or dask array and metadata 
+        Tuple[array6d, mdata, dim_string]: output as 6D numpy or dask array, metadata and dimstring
     """
 
+    if isinstance(filepath, Path):
+        # convert to string
+        filepath = str(filepath)
+
     # get the complete metadata at once as one big class
-    mdata = czimd.CziMetadata(filename)
+    mdata = czimd.CziMetadata(filepath)
 
     if not mdata.consistent_pixeltypes:
         print("Detected PixelTypes ar not consistent. Cannot create array6d")
-        return None
+        return None, mdata, ""
+
     if mdata.consistent_pixeltypes:
         # use pixel type from first channel
         use_pixeltype = mdata.npdtype[0]
@@ -55,14 +64,14 @@ def read_6darray(filename: str,
 
     if output_order not in valid_order:
         print("Invalid dimension order 6D:", output_order)
-        return np.array([]), mdata, ""
+        return None, mdata, ""
 
     if not mdata.scene_shape_is_consistent:
         print("Scenes have inconsistent shape. Cannot read 6D array")
-        return np.array([]), mdata, ""
+        return None, mdata, ""
 
     # open the CZI document to read the
-    with pyczi.open_czi(filename) as czidoc:
+    with pyczi.open_czi(filepath) as czidoc:
 
         if mdata.image.SizeS is not None:
             # get size for a single scene using the 1st
@@ -75,7 +84,6 @@ def read_6darray(filename: str,
             size_y = czidoc.scenes_bounding_rectangle[0].h
 
         if mdata.image.SizeS is None:
-
             # use the size of the total_bounding_rectangle
             size_x = czidoc.total_bounding_rectangle.w
             size_y = czidoc.total_bounding_rectangle.h
@@ -150,18 +158,18 @@ def read_6darray(filename: str,
     return array6d, mdata, dim_string
 
 
-def read_5darray(filename: str,
+def read_5darray(filepath: Union[str, os.PathLike[str]],
                  scene: int = 0,
                  output_order: str = "TCZYX",
                  output_dask: bool = False,
                  chunks_auto: bool = False,
                  remove_adim: bool = True,
-                 **kwargs: int) -> Tuple[Union[np.ndarray, da.Array], czimd.CziMetadata, str]:
+                 **kwargs: int) -> Tuple[Optional[Union[np.ndarray, da.Array]], czimd.CziMetadata, str]:
     """Read a CZI image file as 5D numpy or dask array.
     Important: Currently supported are only scenes with equal size and CZIs with consistent pixel types.
 
     Args:
-        filename (str): filepath for the CZI image
+        filepath (str | Path): filepath for the CZI image
         scene (int, optional): Index of scene to be read as 5D stack.
         output_order (str, optional): Order of dimensions for the output array. Defaults to "TCZYX".
         output_dask (bool, optional): If True the output will be a dask array. Defaults to False.
@@ -174,12 +182,16 @@ def read_5darray(filename: str,
         Tuple[np.ndarray, czimd.Metadata, dim_order6d]: output as 6D numpy or dask array and metadata
     """
 
+    if isinstance(filepath, Path):
+        # convert to string
+        filepath = str(filepath)
+
     # get the complete metadata at once as one big class
-    mdata = czimd.CziMetadata(filename)
+    mdata = czimd.CziMetadata(filepath)
 
     if not mdata.consistent_pixeltypes:
-        print("Detected PixelTypes ar not consitent. Cannot create array6d")
-        return None
+        print("Detected PixelTypes ar not consistent. Cannot create array6d")
+        return None, mdata, ""
     if mdata.consistent_pixeltypes:
         # use pixel type from first channel
         use_pixeltype = mdata.npdtype[0]
@@ -188,10 +200,10 @@ def read_5darray(filename: str,
 
     if output_order not in valid_order:
         print("Invalid dimension order:", output_order)
-        return np.array([]), mdata, ""
+        return None, mdata, ""
 
     # open the CZI document to read the
-    with pyczi.open_czi(filename) as czidoc:
+    with pyczi.open_czi(filepath) as czidoc:
 
         if mdata.image.SizeS is not None:
 
@@ -199,7 +211,7 @@ def read_5darray(filename: str,
             size_x = czidoc.scenes_bounding_rectangle[scene].w
             size_y = czidoc.scenes_bounding_rectangle[scene].h
 
-        if mdata.image.SizeS is None:
+        elif mdata.image.SizeS is None:
 
             # use the size of the total_bounding_rectangle
             size_x = czidoc.total_bounding_rectangle.w
@@ -210,14 +222,12 @@ def read_5darray(filename: str,
         size_z = misc.check_dimsize(mdata.image.SizeZ, set2value=1)
         size_t = misc.check_dimsize(mdata.image.SizeT, set2value=1)
 
-        # define the dimension order to be TZCYXA
-        dimstring = "TZCYXA"
-
         shape = [size_t, size_z, size_c, size_y, size_x, 3 if mdata.isRGB else 1]
 
         if not output_dask:
             array5d = np.empty(shape, dtype=use_pixeltype)
-        if output_dask:
+
+        elif output_dask:
             if chunks_auto:
                 array5d = da.empty(shape, dtype=use_pixeltype, chunks="auto")
             if not chunks_auto:
@@ -234,7 +244,7 @@ def read_5darray(filename: str,
             else:
                 image2d = czidoc.read(plane={'T': t, 'Z': z, 'C': c}, scene=scene)
 
-            # insert 2D image plane into the array6d
+            # insert 2D image plane into the array
             array5d[t, z, c, ...] = image2d
 
             # change the dimension order if needed
@@ -252,20 +262,24 @@ def read_5darray(filename: str,
     return array5d, mdata, dim_string
 
 
-# EXPERIMENTAL FUNCTION
-def read_mdarray_lazy(filename: str, remove_adim: bool = True) -> Tuple[da.Array, str]:
+def read_mdarray_lazy(filepath: Union[str, os.PathLike[str]],
+                      remove_adim: bool = True) -> Tuple[Optional[da.Array], czimd.CziMetadata, str]:
 
-    def read_5d(filename: str,
+    def read_5d(filepath: Union[str, os.PathLike[str]],
                 sizes: Tuple[int, int, int, int, int],
                 s: int,
                 mdata: czimd.CziMetadata,
                 remove_adim: bool = True) -> np.ndarray:
 
+        if isinstance(filepath, Path):
+            # convert to string
+            filepath = str(filepath)
+
         array5d = da.empty([sizes[0], sizes[1], sizes[2], sizes[3], sizes[4], 3 if mdata.isRGB else 1],
                            dtype=mdata.npdtype)
 
         # open the CZI document to read the
-        with pyczi.open_czi(filename) as czidoc5d:
+        with pyczi.open_czi(filepath) as czidoc5d:
 
             # read array for the scene
             for t, z, c in product(range(sizes[0]),
@@ -289,27 +303,29 @@ def read_mdarray_lazy(filename: str, remove_adim: bool = True) -> Tuple[da.Array
 
         return array5d
 
+    if isinstance(filepath, Path):
+        # convert to string
+        filepath = str(filepath)
+
     # get the metadata
-    mdata = czimd.CziMetadata(filename)
+    mdata = czimd.CziMetadata(filepath)
 
     if not mdata.consistent_pixeltypes:
-        print("Detected PixelTypes ar not consitent. Cannot create array6d")
-        return None
+        print("Detected PixelTypes ar not consistent. Cannot create array6d")
+        return None, mdata, ""
     if mdata.consistent_pixeltypes:
         # use pixel type from first channel
         use_pixeltype = mdata.npdtype[0]
 
     # open the CZI document to read the
-    with pyczi.open_czi(filename) as czidoc:
+    with pyczi.open_czi(filepath) as czidoc:
 
         if mdata.image.SizeS is not None:
-
             # use the size of the 1st scenes_bounding_rectangle
             size_x = czidoc.scenes_bounding_rectangle[0].w
             size_y = czidoc.scenes_bounding_rectangle[0].h
 
         if mdata.image.SizeS is None:
-
             # use the size of the total_bounding_rectangle
             size_x = czidoc.total_bounding_rectangle.w
             size_y = czidoc.total_bounding_rectangle.h
@@ -331,7 +347,7 @@ def read_mdarray_lazy(filename: str, remove_adim: bool = True) -> Tuple[da.Array
     # create dask stack of lazy image readers
     lazy_process_image = dask.delayed(read_5d)  # lazy reader
     lazy_arrays = [lazy_process_image(
-        filename, sizes, s, mdata, remove_adim) for s in range(size_s)]
+        filepath, sizes, s, mdata, remove_adim) for s in range(size_s)]
 
     dask_arrays = [da.from_delayed(
         lazy_array, shape=sp, dtype=use_pixeltype) for lazy_array in lazy_arrays]
@@ -340,9 +356,9 @@ def read_mdarray_lazy(filename: str, remove_adim: bool = True) -> Tuple[da.Array
     array_md = da.stack(dask_arrays, axis=0)
 
     # define the dimension order to be STZCYXA
-    dimstring = "STZCYXA"
+    dim_string = "STZCYXA"
 
     if remove_adim:
-        dimstring = dimstring.replace("A", "")
+        dim_string = dim_string.replace("A", "")
 
-    return array_md, dimstring
+    return array_md, mdata, dim_string
